#!/usr/bin/env python
#----------------------------------------------------------------------------
# Project Name: MOSS
# File Name: moss.py
# Description: run server.
#
# Created: 2004. 06. 20
# RCS-ID: $Id: moss.py,v 1.47 2004/07/01 00:41:44 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# Licence: GNU General Public License
# Author: Myung-Gon, Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

__package__ = "MOSS"
__version__ = "0.1"

import ConfigParser
import sys
import getopt
import template
import web
import live
import os

try:
	import ogg.vorbis
	ogg_support = "yes"
except ImportError:
	ogg_support = "no"

try:
	import signal
	signal_support = "yes"
except ImportError:
	signal_support = "no"

def sigterm_handler(signum, frame):
	global running
	running = None
	live.running = None
	
def sighup_handler(signum, frame):
	global reconfig
	reconfig = 1
	live.running = None
	
def daemonize(stdin = "/dev/null", stdout = "/dev/null", stderr = "/dev/null"):
	try: 
		pid = os.fork() 
		if pid > 0:
			sys.exit(0) # Exit first parent.
	except OSError, error: 
		sys.stderr.write("fork #1 failed: (%d) %s\n" % (error.errno, error.strerror)    )
		sys.exit(1)
        
	os.chdir("/") 
	os.umask(0) 
	os.setsid() 
   
	try: 
		pid = os.fork() 
		if pid > 0:
			sys.exit(0) # Exit second parent.
	except OSError, error: 
		sys.stderr.write("fork #2 failed: (%d) %s\n" % (error.errno, error.strerror)    )
		sys.exit(1)

	si = open(stdin, "r")
	so = open(stdout, "a+")
	se = open(stderr, "a+", 0)
	
	os.dup2(si.fileno(), sys.stdin.fileno())
	os.dup2(so.fileno(), sys.stdout.fileno())
	os.dup2(se.fileno(), sys.stderr.fileno())

def run_server(config_file, server_mode):
	global signal_support
	global reconfig
	global running 
	
	reconfig = None
	
	if signal_support == "yes":
		if "SIGTERM" in dir(signal):
			signal.signal(signal.SIGTERM, sigterm_handler)
		if "SIGHUP" in dir(signal):
			signal.signal(signal.SIGHUP, sighup_handler)
	
	if server_mode == "web":
		server = web.Server(config_file)
		server.log_message("MOSS: Server started with pid %d" % os.getpid())
		running = 1
		
		try:
			while running:
				if reconfig:
					server.log_message("MOSS: Reloading configuraion file")
					server.server_close()
					server = web.Server(config_file)
					reconfig = None

				server.handle_request()

			server.log_message("MOSS: Now taking down the server...")
			sys.exit(0)
		
		except KeyboardInterrupt:
			server.log_message("MOSS: KeyboardInterrupt occured")
			server.log_message("MOSS: Please wait while the remaining streams finish...")
			sys.exit(0)
	else:
		server = live.Server(config_file)
		server.log_message("MOSS: Server started with pid %d" % os.getpid())
		running = 1
		
		while running:
			if reconfig:
				server = live.Server(config_file)
				server.log_message("MOSS: Reloading configuration file")
				reconfig = None

			server.serve_forever()
		
		server.log_message("MOSS: Now taking down the server...")
		sys.exit(0)
		
def print_usage():
	print "Usage: %s [--help] [--debug] [--config <file name>]" % os.path.basename(sys.argv[0])
	print "       -h      --help      Display this usage information."
	print "       -d      --debug     Run as a debug mode."
	print "       -c      --config    Specify the config file(default: moss.conf)"

if __name__ == "__main__":
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hdc:", ["help", "debug", "config"])
	except getopt.GetoptError:
		print_usage()
		sys.exit(1)
	
	debug_mode = None
	config_file = "moss.conf"

	for opt, arg in opts:
		if opt in ("-h", "--help"):
			print_usage()
			sys.exit(0)
		
		if opt in ("-d", "--debug"):
			debug_mode = 1
		
		if opt in ("-c", "--config"):
			config_file = arg
			
			if os.path.isfile(config_file):
				print "MOSS: No such configuration file"
				print_usage()
				sys.exit(1)

	if os.path.isfile(config_file):
		config = ConfigParser.ConfigParser()
		config.add_section("server")
		config_defaults = config.defaults()
		config_defaults["server-mode"] = "web"
		config.read(config_file)
		server_mode = config.get("server", "server-mode")
	
		if server_mode == "web":
			config = ConfigParser.ConfigParser()
			config.add_section("extra")
			config_defaults = config.defaults()
			config_defaults["template-dir"] = "templates"
			config_defaults["template"] = "default.tmpl"
			config.read(config_file)
			template_path = config.get("extra", "template-dir")
			template_file = config.get("extra", "template")
			template_file = os.path.join(template_path, template_file)

			if os.path.isfile(template_file):
				template_file = os.path.realpath(template_file)
				web.template = template.Template(template_file)
			else:
				print "MOSS: No such template file"
				sys.exit(1)

			web.debug_mode = debug_mode
		elif server_mode == "live":
			live.debug_mode = debug_mode
		else:
			print "MOSS: Invalid server mode"
			sys.exit(1)
			
	else:
		print "MOSS: Configuration file not found"
		sys.exit(1)
	
	if ogg_support == "yes":
		print "MOSS: Ogg vorbis support enabled"
	else:
		print "MOSS: Ogg vorbis support disabled, to enable it you will need to install the 'pyogg' and the 'pyvorbis' modules"
			
	if not debug_mode:
		print "MOSS: Entering in daemon mode"
		config_file = os.path.realpath(config_file)
		daemonize("/dev/null", "/tmp/moss.log", "/tmp/moss.log")
	else:
		print "MOSS: Entering in debug mode"
	
	run_server(config_file, server_mode)
			
